/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.clustering.ClusterManager;
import org.apache.axis2.clustering.configuration.ConfigurationManager;
import org.apache.axis2.clustering.context.ContextManager;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.adminui.AdminUIServletFilter;
import org.wso2.utils.FileManipulator;
import org.wso2.utils.ServerConfiguration;
import org.wso2.utils.ServerException;
import org.wso2.utils.transport.ProxyCache;
import org.wso2.wsas.serverinfo.ServerInfo;
import org.wso2.wsas.transport.ServerPropertyKeys;
import org.wso2.wsas.util.ClusteringUtil;
import org.wso2.wsas.util.Controllable;
import org.wso2.wsas.util.MIMEType2FileExtensionMap;
import org.wso2.wsas.util.Monitor;
import org.wso2.wsas.util.ServerController;
import org.wso2.wsas.util.SystemRestarter;
import org.wso2.wsas.util.Utils;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

public class MainServlet extends HttpServlet implements Controllable {
    private static final Log log = LogFactory.getLog(MainServlet.class);

    private static final long serialVersionUID = 4517849288538613640L;

    protected ServerManager serverManager;
    private ServletConfig servletConfig;
    private String wso2wsasHome;
    private static ServerConfiguration serverConfig;
    private Thread shutdownHook;
    private static ServerController serverController;
    protected String serverName;

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response) throws ServletException,
                                                              IOException {
    }

    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response) throws ServletException,
                                                               IOException {
    }

    public void init(ServletConfig servletConfig) throws ServletException {
        long startTime;
        if (System.getProperty(ServerConstants.WSO2WSAS_START_TIME) == null) {
            startTime = System.currentTimeMillis();
        } else {
            startTime = Long.parseLong(System.getProperty(ServerConstants.WSO2WSAS_START_TIME));
        }

        this.servletConfig = servletConfig;

        // Expanding of web content within AAR file
        try {
            System.setProperty(ServerConstants.WEB_RESOURCE_LOCATION,
                               servletConfig.getServletContext().getRealPath("/wservices"));
        } catch (Exception e) {
            log.warn("wso2wsas.war is not expanded. Web content within AAR file not supported.");
        }

        ServerInfo serverInfo;
        try {
            InputStream is =
                    Thread.currentThread().getContextClassLoader().
                            getResourceAsStream("wso2wsas.properties");
            Properties props = new Properties();
            if (is != null) {
                props.load(is);
            } else {
                throw new RuntimeException("wso2wsas.properties file not found in classpath");
            }

            //Setting the WSAS home
            wso2wsasHome = System.getProperty(ServerConstants.WSO2WSAS_HOME);
            if (wso2wsasHome == null) {
                wso2wsasHome = props.getProperty(ServerConstants.WSO2WSAS_HOME);
                if (wso2wsasHome == null) {
                    log.fatal(ServerConstants.WSO2WSAS_HOME +
                              " property not set in wso2wsas.properties file");
                    log.fatal(serverName + " startup failed.");
                    return;
                }
            }

            System.setProperty(ServerConstants.AXIS2_HOME, wso2wsasHome);
            System.setProperty(ServerConstants.WSO2WSAS_HOME, wso2wsasHome);

            // -------------------------------------------------------------------------------------
            serverConfig = ServerConfiguration.getInstance();
            serverConfig.init(Utils.getWsasServerXml());
            serverManager = ServerManager.getInstance();
            // -------------------------------------------------------------------------------------

            serverName = serverConfig.getFirstProperty("Name");

            File file = new File(wso2wsasHome);
            if (!file.exists()) {
                log.fatal(ServerConstants.WSO2WSAS_HOME + " " + wso2wsasHome +
                          " does not exist.");
                log.info("WSO2 WSAS startup failed.");
                return;
            }
            if (!file.isDirectory()) {
                log.fatal(ServerConstants.WSO2WSAS_HOME + " " + wso2wsasHome +
                          " is not a directory.");
                log.fatal(serverName + " startup failed.");
                return;
            }

            is.close();

            // ServerInfo class
            String serverInfoImplClass = props.getProperty(ServerPropertyKeys.APPSERVER_INFO_IMPL);
            log.info("Using Server Info Class: " + serverInfoImplClass);
            serverInfo = (ServerInfo) Class.forName(serverInfoImplClass).newInstance();
            serverInfo.setProperties(props);
            is.close();
        } catch (IOException e) {
            log.error("wso2wsas.properties file cannot be read", e);
            log.fatal(serverName + " startup failed.");
            return;
        } catch (Exception e) {
            log.error("Exception occurred while initializing MainServlet : " + e);
            log.fatal(serverName + " startup failed.");
            return;
        }

        try {
            start(servletConfig);
            serverInfo.process();
            ServerManager.getInstance().setHttpPort(serverInfo.getHttpPort());
            ServerManager.getInstance().setHttpsPort(serverInfo.getHttpsPort());
            ConfigurationContext configCtx = serverManager.configContext;
            Utils.setServletContextProperties(servletConfig.getServletContext(),
                                              (Map) configCtx
                                                      .getProperty(ServerConstants.GENERATED_PAGES),
                                              Utils.isAdminConsoleEnabled(),
                                              ProxyCache.getInstance().getHttpPort() != -1 ?
                                              ProxyCache.getInstance().getHttpPort() :
                                              ServerManager.getInstance().getHttpPort(),
                                              ProxyCache.getInstance().getHttpsPort() != -1 ?
                                              ProxyCache.getInstance().getHttpsPort() :
                                              ServerManager.getInstance().getHttpsPort(),
                                              configCtx.getServicePath());
            printInfo(startTime);
            System.getProperties().remove(ServerConstants.WSO2WSAS_START_TIME);
        } catch (ServerException e) {
            log.error(e);
        }
    }

    public void start(ServletConfig servletConfig) throws ServerException {
        ServletContext servletContext = servletConfig.getServletContext();

        System.setProperty(ServerConstants.DERBY_HOME,
                           new File(serverConfig.getFirstProperty(
                                   "Database.Home")).getAbsolutePath());
        serverManager.adminResourceBase = servletContext.getRealPath(".");
        if (serverManager.adminResourceBase == null) {
            serverManager.adminResourceBase = "TODO"; //TODO: Get the path to the resource base. Handle unexpanded case.
        }
        serverManager.serverWorkDir =
                new File(serverConfig.getFirstProperty("WorkDirectory")).getAbsolutePath();
        System.setProperty("axis2.work.dir", serverManager.serverWorkDir);

        serverManager.axis2RepoLocation =
                serverConfig.getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
        if (!serverManager.axis2RepoLocation.endsWith("/")) {
            serverConfig.setConfigurationProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION,
                                                  serverManager.axis2RepoLocation + "/");
            serverManager.axis2RepoLocation = serverManager.axis2RepoLocation + "/";
        }
        System.setProperty(Constants.AXIS2_REPO, serverManager.axis2RepoLocation);

        MIMEType2FileExtensionMap.mappingFileName = wso2wsasHome + File.separator +
                                                    "conf" + File.separator + "mime-mappings.xml";

        try {
            serverManager.start();
            addShutdownHook();
            ConfigurationContext configCtx = serverManager.configContext;
            servletConfig.getServletContext().setAttribute(ServerConstants.CONFIGURATION_CONTEXT,
                                                           configCtx);
            configCtx.setProperty(Constants.CONTAINER_MANAGED, "true");
            configCtx.setProperty(ServerConstants.WORK_DIR, serverManager.serverWorkDir);

            configCtx.setProperty(ServerConstants.WSO2WSAS_INSTANCE, this);
            startServerController();
        } catch (Exception e) {
            throw new ServerException(serverName + " startup failed.", e);
        }
    }

    private void printInfo(long startTime) {
        log.info("Using Repository " + serverManager.axis2RepoLocation);
        log.info("");
        log.info("Listening for HTTP on             ----> " +
                 ServerManager.getInstance().getHttpPort());
        log.info("Listening for HTTPS on            ----> " +
                 ServerManager.getInstance().getHttpsPort());
        log.info("Management console is accessible only via the HTTPS URL");
        log.info("");
//        serverManager.printMgtConsoleURL();
        log.info(serverName + " started in " + (System.currentTimeMillis() - startTime) + " ms");
    }

    private void addShutdownHook() {
        if (shutdownHook != null) {
            return;
        }

        final MainServlet main = this;
        shutdownHook = new Thread() {
            public void run() {
                log.info("Shutting down " + serverName + "...");
                try {
                    main.shutdown();
                    log.info("Shutdown complete");
                    log.info("Halting JVM");
                } catch (Exception e) {
                    log.warn("Error occurred while shutting down WSO2 WSAS : " + e);
                }
            }
        };
        Runtime.getRuntime().addShutdownHook(shutdownHook);
    }

    public void setAxis2RepoLocation(String axis2RepoLocation) {
        serverManager.axis2RepoLocation = axis2RepoLocation;
    }

    public void setMonitor(Monitor monitor) {
        serverManager.monitor = monitor;
    }

    public void startServer() throws ServerException {
        try {
            init(this.servletConfig);
            reinitializeServlets(this.servletConfig.getServletContext());
        } catch (ServletException e) {
            String msg = "Cannot start server";
            log.fatal(msg, e);
            throw new ServerException(msg, e);
        }
    }

    public void stopListeners() throws ServerException {
    }

    public void restartServer() {
        SystemRestarter st = new SystemRestarter(this,
                                                 this,
                                                 serverManager.axis2RepoLocation,
                                                 serverManager.monitor);
        cleanupSystem();
        new Thread(st).start();
    }

    public Thread getShutdownHook() {
        return shutdownHook;
    }

    public void setConfigurationContext(ConfigurationContext newConfigCtx) throws
                                                                           ServerException {
        try {

            // We need to preserve the old ClusterManager
            ConfigurationContext oldConfigCtx = serverManager.configContext;
            AxisConfiguration oldAxisConfig = oldConfigCtx.getAxisConfiguration();
            ClusterManager originalClusterManager = oldAxisConfig.getClusterManager();
            newConfigCtx.getAxisConfiguration().setClusterManager(originalClusterManager);

            // We need to preserve the original properties which were in the config ctx
            for (Iterator iter = oldConfigCtx.getPropertyNames();
                 iter.hasNext();) {
                String key = (String) iter.next();
                newConfigCtx.setProperty(key, oldConfigCtx.getProperty(key));
            }

            // Remove all clustering references
            originalClusterManager.setConfigurationContext(newConfigCtx);
            ConfigurationManager configurationManager =
                    originalClusterManager.getConfigurationManager();
            if (configurationManager != null) {
                configurationManager.setConfigurationContext(newConfigCtx);
            }
            ContextManager contextManager = originalClusterManager.getContextManager();
            if (contextManager != null) {
                contextManager.setConfigurationContext(newConfigCtx);
            }
            oldAxisConfig.setClusterManager(null);

            // Stop the ListenerManager
            try {
                serverManager.stopListenerManager();
            } catch (AxisFault axisFault) {
                throw new ServerException(axisFault);
            }

            // Now we switch to the new Config ctx
            serverManager.configContext = newConfigCtx;

            // Disable admin services, if needed
            if (!Utils.isAdminConsoleEnabled()) {
                newConfigCtx.getAxisConfiguration().
                        removeServiceGroup(ServerConstants.ADMIN_SERVICE_GROUP);
            }

            // Start the ListenerManager
            serverManager.startListenerManager();

            ServletContext servletContext = servletConfig.getServletContext();
            servletContext.removeAttribute(ServerConstants.CONFIGURATION_CONTEXT);
            servletContext.setAttribute(ServerConstants.CONFIGURATION_CONTEXT,
                                        newConfigCtx);
            reinitializeServlets(servletContext);

            // Need to enable clustering on the new configurationContext
            try {
                ClusteringUtil.enableClustering(newConfigCtx);
            } catch (AxisFault axisFault) {
                throw new ServerException(axisFault);
            }
        } catch (Exception e) {
            String msg = "Cannot set ConfigurationContext";
            log.fatal(msg, e);
            throw new ServerException(msg, e);
        }
    }

    private void reinitializeServlets(ServletContext servletContext) throws ServletException {
        // Reinitialize all of the servlets, for this puprpose,
        // the servlets should have added themselves
        // as ServletContext attributes in the previous init call
        Enumeration attributeNames = servletContext.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object attrib = servletContext.getAttribute((String) attributeNames.nextElement());
            if (attrib instanceof HttpServlet) {
                ((HttpServlet) attrib).init();
            } else if (attrib instanceof AdminUIServletFilter) {
                AdminUIServletFilter adminUIServletFilter = (AdminUIServletFilter) attrib;
                adminUIServletFilter.init();
            }
        }
    }

    public void shutdown() throws ServerException {
        cleanupSystem();
        //calling ListenerManager.stop
        try {
            serverManager.stopListenerManager();
        } catch (AxisFault axisFault) {
            throw new ServerException(axisFault);
        }
    }

    private void startServerController() {
        if (serverController != null) {
            return;
        }
        serverController = new ServerController(this);
        String cmdListenerPortStr =
                serverConfig.getFirstProperty(ServerConfiguration.COMMAND_LISTENER_PORT);
        if (cmdListenerPortStr == null) {
            return;
        }
        int cmdListenerPort = Integer.parseInt(cmdListenerPortStr);
        serverController.setPort(cmdListenerPort);
        Thread thread = new Thread(serverController);
        thread.start();
        log.info("Command listener starting on port ----> " + cmdListenerPort);
    }

    public void destroy() {
        try {
            shutdown();
        } catch (ServerException e) {
            log.error(e);
        }
    }

    private void cleanupSystem() {
        log.info("Cleaning up system...");
        new FileManipulator().deleteDir(new File(wso2wsasHome + File.separator +
                                                 serverConfig.
                                                         getFirstProperty("WorkDirectory")));
        ConfigurationContext configContext = serverManager.configContext;
        if (configContext != null) {
            Object property = configContext.getProperty(ServerConstants.FILE_RESOURCE_MAP);
            if (property != null) {
                ((Map) property).clear();
            }
        }
    }
}
