/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.transport.http;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.catalina.connector.Connector;
import org.apache.catalina.startup.Embedded;
import org.wso2.utils.NetworkUtils;
import org.wso2.utils.transport.AbstractTransportListener;
import org.wso2.utils.transport.ProxyCache;
import org.wso2.wsas.ServerConstants;
import org.wso2.wsas.TomcatServerCache;

import java.net.InetAddress;
import java.net.SocketException;

/**
 *
 */
public class HttpTransportListener extends AbstractTransportListener {

    public EndpointReference getEPRForService(String serviceName,
                                              String ip) throws AxisFault {
        return getEPR(ServerConstants.HTTP_TRANSPORT, serviceName, ip);
    }

    public EndpointReference[] getEPRsForService(String serviceName,
                                                 String ip) throws AxisFault {
        return new EndpointReference[]{getEPRForService(serviceName, ip)};
    }

    public void start() throws AxisFault {
        try {
            Embedded embedded = TomcatServerCache.getEmbedded();
            Connector httpConnector =
                    embedded.createConnector((InetAddress) null, port, false);
            httpConnector.setEnableLookups(true);

            if (TomcatServerCache.addConnector(httpConnector)) {
                httpConnector.start();
            }
            ProxyCache.getInstance().setHttpPort(proxyPort);
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
    }
}
