/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.util;

import org.apache.axis2.Constants;
import org.apache.axis2.context.ConfigurationContext;
import org.wso2.adminui.AdminUIConstants;
import org.wso2.utils.ServerConfiguration;
import org.wso2.wsas.ServerConstants;
import org.wso2.wsas.ServerManager;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

/**
 *
 */
public class Utils {

    public static boolean isAdminConsoleEnabled() {
        boolean enableAdminConsole = false;
        String enableAdminConsoleProp =
                ServerConfiguration.getInstance().getFirstProperty("Management.EnableConsole");
        if (enableAdminConsoleProp != null) {
            enableAdminConsole = Boolean.valueOf(enableAdminConsoleProp).booleanValue();
        }
        return enableAdminConsole;
    }

    public static void setServletContextProperties(ServletContext servletContext,
                                                   Map generatedPages,
                                                   boolean isAdminConsoleEnabled,
                                                   int httpPort,
                                                   int httpsPort,
                                                   String servicePath)
            throws ServletException {
        servletContext.setAttribute(AdminUIConstants.GENERATED_PAGES, generatedPages);
        servletContext.setAttribute(AdminUIConstants.ENABLE_CONSOLE,
                                    Boolean.valueOf(isAdminConsoleEnabled));
        servletContext.setAttribute(AdminUIConstants.HTTP_PORT, httpPort + "");
        servletContext.setAttribute(AdminUIConstants.HTTPS_PORT, httpsPort + "");
        servletContext.setAttribute(AdminUIConstants.SERVICE_CONTEXT_PATH, servicePath);
    }

    public static String getAxis2Xml() {
        String axis2XML = ServerConfiguration.getInstance().
                getFirstProperty("Axis2Config.ConfigurationFile");
        if (axis2XML == null) {
            axis2XML = System.getProperty(Constants.AXIS2_CONF);
        }
        return axis2XML;
    }

    public static String getWsasServerXml() {
        String serverXML = System.getProperty(ServerConstants.WSO2WSAS_SERVER_XML);
        if (serverXML == null) {
            serverXML =
                    System.getProperty(ServerConstants.WSO2WSAS_HOME) + File.separator + "conf" +
                    File.separator + "server.xml";
        }
        return serverXML;
    }

    public static boolean isURL(String location) {
        try {
            new URL(location);
            return true;
        } catch (MalformedURLException e) {
            return false;
        }
    }
}
